@@ -67,12 +67,36 @@ last_temp_log = nil
 last_temp_value = 0
 
 function logtemp( temp )
-	now = os.time()
-	-- Log only every 5 minutes or if the temperature has changed by
-	-- more than 5.
-	if ( ( not last_temp_log ) or
-	     ( os.difftime(now, last_temp_log) >= 300 ) or
-	     ( math.abs( temp - last_temp_value ) >= 5 ) ) then
+	local now = os.time()
+
+	local function should_log(  )
+		local delta_temp = math.abs(temp - last_temp_value)
+		local delta_time = os.difftime(now, last_temp_log)
+
+		-- Haven't previously logged, log now for the first time
+		if ( not last_temp_log ) then
+			return true
+		end
+		-- Temperature has changed by more than 5
+		if ( delta_temp >= 5 ) then
+			return true
+		end
+		-- More than 5 minutes have elapsed...
+		if ( delta_time >= 300 ) then
+			if ( delta_temp > 1 ) then
+				--- ...and the change is by more than +/-1
+				return true
+			else
+				--- ...insignificant change, wait another 5 minutes
+				last_temp_log = now
+				return false
+			end
+		end
+		-- Otherwise no need to log
+		return false
+	end
+
+	if ( should_log() ) then
 		logprint(string.format("ts219: temperature %d", temp))
 		last_temp_log = now
 		last_temp_value = temp
@@ -81,10 +105,10 @@ end
 
 last_fan_setting = nil
 
-function setfan( speed )
+function setfan( temp, speed )
 	if ( ( not last_fan_setting ) or
 	     ( last_fan_setting ~= speed ) ) then
-		logprint(string.format("ts219: setting fan to \"%s\"", speed))
+		logprint(string.format("ts219: temperature %d setting fan to \"%s\"", temp, speed))
 	end
 	piccmd("fanspeed", speed)
 	last_fan_setting = speed
@@ -98,33 +122,38 @@ function temp( temp )
 	logtemp(temp)
 	if last_fan_setting == "full" then
 		if temp < 55 then
-			setfan("high")
+			setfan(temp, "high")
 		end
 	elseif last_fan_setting == "high" then
 		if temp > 65 then
-			setfan("full")
+			setfan(temp, "full")
 		elseif temp < 45 then
-			setfan("medium")
+			setfan(temp, "medium")
 		end
 	elseif last_fan_setting == "medium" then
 		if temp > 50 then
-			setfan("high")
+			setfan(temp, "high")
 		elseif temp < 35 then
-			setfan("low")
+			setfan(temp, "low")
 		end
 	elseif last_fan_setting == "low" then
 		if temp > 40 then
-			setfan("medium")
+			setfan(temp, "medium")
 		elseif temp < 32 then
-			setfan("silence")
+			setfan(temp, "silence")
 		end
 	elseif last_fan_setting == "silence" then
 		if temp > 35 then
-			setfan("low")
+			setfan(temp, "low")
 		end
 	else
-		setfan("high")
+		setfan(temp, "high")
 	end
 end
 
 confdir("/etc/qcontrol.d")
+--
+-- Local variables:
+-- mode: lua
+-- indent-level: 8
+-- End:
