@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2007-2008  Byron Bradley (byron.bbradley@gmail.com)
+ * Copyright (C) 2013  Ian Campbell (ijc@hellion.org.uk)
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <getopt.h>
 #include <glob.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -58,13 +60,15 @@ char *usage = "Usage: qcontrol [OPTION...] [command] [args...]\n"
               "PIC Controller\n\n"
               "  -d, --daemon               Run the server as a daemon\n"
               "  -f, --foreground           Run the server in the foreground\n"
-              "  -?, --help                 Give this help list\n"
+              "      --direct               Run commands directly\n"
+              "  -c, --config=PATH          Use PATH as config file\n"
+              "      --help                 Give this help list\n"
               "  -V, --version              Print program version\n\n"
               "Mandatory or optional arguments to long options are also "
               "mandatory or optional\nfor any corresponding short options.\n";
 static const char *configfilename = "/etc/qcontrol.conf";
 static lua_State *lua = NULL;
-unsigned int commandcount;
+unsigned int commandcount = 0;
 struct piccommand **commands;
 
 extern struct picmodule system_module;
@@ -74,6 +78,7 @@ extern struct picmodule ts409_module;
 extern struct picmodule ts41x_module;
 extern struct picmodule a125_module;
 extern struct picmodule evdev_module;
+extern struct picmodule synology_module;
 
 struct picmodule *modules[] = {
 	&system_module,
@@ -83,6 +88,7 @@ struct picmodule *modules[] = {
 	&ts41x_module,
 	&a125_module,
 	&evdev_module,
+	&synology_module,
 	NULL
 };
 
@@ -165,7 +171,8 @@ static int confdir(lua_State *L UNUSED)
 	const char *path = lua_tostring(lua, 1);
 	glob_t gl;
 	char *gl_path;
-	int rc, i;
+	size_t p;
+	int rc;
 
 	if (!path) {
 		print_log(LOG_ERR, "confdir: no path given");
@@ -200,10 +207,10 @@ static int confdir(lua_State *L UNUSED)
 		return -1;
 	}
 
-	for (i=0; i<gl.gl_pathc; i++) {
+	for (p=0; p<gl.gl_pathc; p++) {
 		struct stat st_buf;
 
-		path = gl.gl_pathv[i];
+		path = gl.gl_pathv[p];
 
 		rc = stat(path, &st_buf);
 		if (rc < 0) {
@@ -295,28 +302,34 @@ int register_command(const char *cmd, const char *shorthelp, const char *help,
 	return 0;
 }
 
+static int shorthelp_fmt(char *buf, size_t size, struct piccommand *c)
+{
+	return snprintf(buf, size, "%-16s %s\n", c->name, c->shorthelp);
+}
+
 static void shorthelp_commands(char **buf, int *len)
 {
 	unsigned int i, off=0;
 
-	*len = 0;
+	*len = sizeof(int) + 1; /* Length field + NUL terminator */
 	for (i = 0; i < commandcount; ++i)
-		*len += strlen(commands[i]->shorthelp) + MAX_CMD_NAME
-		        + sizeof("\n") + sizeof(int);
+		*len += shorthelp_fmt(NULL, 0, commands[i]);
 
 	*buf = malloc(*len);
 	memset(*buf, 0, sizeof(int));
-	off += sizeof(int);
-	for (i = 0; i < commandcount; ++i) {
-		strncpy(*buf+off, commands[i]->name, MAX_CMD_NAME);
-		off += strlen(commands[i]->name);
-		memset(*buf+off, ' ', MAX_CMD_NAME - strlen(commands[i]->name));
-		off += MAX_CMD_NAME - strlen(commands[i]->name);
-		strncpy(*buf+off, commands[i]->shorthelp, *len-off);
-		off += strlen(commands[i]->shorthelp);
-		strncpy(*buf+off, "\n", *len-off);
-		off += strlen("\n");
-	}
+	off += sizeof(int); /* Skip length */
+	for (i = 0; i < commandcount; ++i)
+		off += shorthelp_fmt(*buf+off, *len-off, commands[i]);
+}
+
+static void shorthelp_commands_direct(void)
+{
+	char *buf = NULL;
+	int offs = 0;
+
+	shorthelp_commands(&buf, &offs);
+	printf("\nAvailable commands are:\n%s", buf+sizeof(int));
+	free(buf);
 }
 
 static int run_command(const char *cmd, int argc, const char **argv)
@@ -344,6 +357,16 @@ static int run_command_lua(lua_State *L UNUSED)
 	return run_command(argv[0], argc-1, argv+1);
 }
 
+static int run_command_direct(const char *cmd, int argc, const char **argv)
+{
+	int err = run_command(cmd, argc, argv);
+
+	if (err < 0)
+		return -1;
+
+	return 0;
+}
+
 static int script_print(lua_State *L UNUSED)
 {
 	int argc, err;
@@ -473,6 +496,7 @@ static int network_send(int argc, const char **argv)
 	}
 
 	send(sock, buf, off, 0);
+	free(buf);
 	rlen = read(sock, retbuf, MAX_NET_BUF);
 	if (rlen < 0) {
 		print_log(LOG_ERR, "Error during read: %s", strerror(errno));
@@ -668,42 +692,79 @@ static int start_daemon(bool daemon_mode)
 	return err;
 }
 
-int main(int argc, const char *argv[])
+int main(int argc, char **argv)
 {
-	const char *help = "--help";
-	commandcount = 0;
+	const char *help_arg = "--help";
+	enum {
+		MODE_SERVER_DAEMON,
+		MODE_SERVER_FOREGROUND,
+		MODE_CLIENT,
+		MODE_DIRECT,
+	} mode = MODE_CLIENT;
+	bool help = false;
+
+	while (1) {
+		struct option long_options[] = {
+			{"config",      required_argument, 0, 'c' },
+			{"daemon",      no_argument,       0, 'd' },
+			{"foreground",  no_argument,       0, 'f' },
+			{"direct",      no_argument,       0, 1  },
+			{"help",        no_argument,       0, 'h' },
+			{"version",     no_argument,       0, 'V' },
+			{0, 0, 0, 0}
+		};
+
+		int opt = getopt_long(argc, argv, "c:dfhv",
+				      long_options, NULL);
+
+		if (opt == -1)
+			break;
+
+		switch (opt) {
+		case 1:   mode = MODE_DIRECT;            break;
+		case 'c': configfilename = optarg;       break;
+		case 'd': mode = MODE_SERVER_DAEMON;     break;
+		case 'f': mode = MODE_SERVER_FOREGROUND; break;
+		case 'h': help = 1;                      break;
+		case 'V': printf("%s", version);         return 0;
+		case '?':                                break;
+		}
 
-	if (argc > 1 && (strcmp(argv[1], "-c") == 0)) {
-		configfilename = argv[2];
-		argc -= 2;
-		argv += 2;
 	}
 
-	if (argc > 1 && (strcmp(argv[1], "--help") == 0
-	              || strcmp(argv[1], "-?") == 0)) {
-		printf("%s", usage);
-		return network_send(argc - 1, argv + 1);
-	} else if (argc > 1 && (strcmp(argv[1], "-V") == 0
-	                     || strcmp(argv[1], "--version") == 0)) {
-		printf("%s", version);
-		return 0;
-	} else if (argc > 1 && (strcmp(argv[1], "-d") == 0
-                                || strcmp(argv[1], "--daemon") == 0)) {
-		return start_daemon(true);
-	} else if (argc > 1 && (strcmp(argv[1], "-f") == 0
-                                || strcmp(argv[1], "--foreground") == 0)) {
-		return start_daemon(false);
-	} else if (argc > 2 && strcmp(argv[1], "--direct") == 0) {
-		/* Execute a single command and terminate */
-		pic_lua_setup(&lua);
-		return run_command(argv[2], argc - 3, argv + 3);
-	} else if (argc > 1) {
+	argc -= optind;
+	argv += optind;
+
+	switch (mode) {
+	case MODE_SERVER_DAEMON:
+	case MODE_SERVER_FOREGROUND:
+		if (help) {
+			printf("%s", usage);
+			return 0;
+		} else {
+			return start_daemon(mode == MODE_SERVER_DAEMON);
+		}
+
+	case MODE_CLIENT:
 		/* Send the command to the server */
-		return network_send(argc - 1, argv + 1);
-	} else {
-		printf("%s", usage);
-		return network_send(1, &help);
+		if (help || argc == 0) {
+			printf("%s", usage);
+			return network_send(1, &help_arg);
+		} else {
+			return network_send(argc, (const char **)argv);
+		}
+
+	case MODE_DIRECT:
+		pic_lua_setup(&lua);
+		if (help || argc == 0) {
+			printf("%s", usage);
+			shorthelp_commands_direct();
+			return 0;
+		} else {
+			/* Execute a single command and terminate */
+			return run_command_direct(argv[0], argc - 1, (const char **)(argv + 1));
+		}
 	}
 
-	return -1;
+	abort(); /* Not reached */
 }
